@@ -1,14 +1,29 @@
-use async_trait::async_trait;
-use std::collections::HashMap;
-use std::fmt::{Display, Formatter};
-use std::sync::{Arc, RwLock};
+use std::{
+    collections::HashMap,
+    sync::{
+        Arc,
+        RwLock,
+    },
+};
 
-use serde::{Deserialize, Serialize};
+use serde::{
+    Deserialize,
+    Serialize,
+};
+use static_assertions::assert_impl_all;
 
-use cqrs_es::mem_store::MemStore;
-use cqrs_es::test::TestFramework;
-use cqrs_es::Query;
-use cqrs_es::{Aggregate, AggregateError, CqrsFramework, DomainEvent, EventEnvelope, EventStore};
+use cqrs_es2::{
+    test::TestFramework,
+    Aggregate,
+    AggregateError,
+    CqrsFramework,
+    DomainEvent,
+    EventEnvelope,
+    EventStore,
+    MemoryStore,
+    MemoryStoreAggregateContext,
+    QueryProcessor,
+};
 
 #[derive(Debug, Serialize, Deserialize)]
 pub struct TestAggregate {
@@ -16,81 +31,64 @@ pub struct TestAggregate {
     description: String,
     tests: Vec<String>,
 }
-#[derive(Debug)]
-pub struct TestError(String);
-impl Display for TestError {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{}", self.0)
-    }
-}
 
-impl std::error::Error for TestError {}
-
-impl From<&str> for TestError {
-    fn from(msg: &str) -> Self {
-        Self(msg.to_string())
-    }
-}
-
-#[derive(Clone, Debug)]
-pub struct TestService;
-
-#[async_trait]
 impl Aggregate for TestAggregate {
     type Command = TestCommand;
     type Event = TestEvent;
-    type Error = TestError;
-    type Services = TestService;
 
-    fn aggregate_type() -> String {
-        "TestAggregate".to_string()
+    fn aggregate_type() -> &'static str {
+        "TestAggregate"
     }
 
-    async fn handle(
+    fn handle(
         &self,
-        command: Self::Command,
-        _service: &Self::Services,
-    ) -> Result<Vec<TestEvent>, Self::Error> {
+        command: TestCommand,
+    ) -> Result<Vec<TestEvent>, AggregateError> {
         match &command {
             TestCommand::CreateTest(command) => {
                 let event = TestEvent::Created(Created {
                     id: command.id.to_string(),
                 });
                 Ok(vec![event])
-            }
+            },
 
             TestCommand::ConfirmTest(command) => {
                 for test in &self.tests {
                     if test == &command.test_name {
-                        return Err("test already performed".into());
+                        return Err(AggregateError::new(
+                            "test already performed",
+                        ));
                     }
                 }
                 let event = TestEvent::Tested(Tested {
                     test_name: command.test_name.to_string(),
                 });
                 Ok(vec![event])
-            }
+            },
 
             TestCommand::DoSomethingElse(command) => {
                 let event = TestEvent::SomethingElse(SomethingElse {
                     description: command.description.clone(),
                 });
                 Ok(vec![event])
-            }
+            },
         }
     }
 
-    fn apply(&mut self, event: Self::Event) {
+    fn apply(
+        &mut self,
+        event: &Self::Event,
+    ) {
         match event {
             TestEvent::Created(e) => {
                 self.id = e.id.clone();
-            }
+            },
             TestEvent::Tested(e) => {
                 self.tests.push(e.test_name.clone());
-            }
+            },
             TestEvent::SomethingElse(e) => {
                 self.description = e.description.clone();
-            }
+            },
         }
     }
 }
@@ -105,41 +103,53 @@ impl Default for TestAggregate {
     }
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
+#[derive(
+    Debug,
+    Serialize,
+    Deserialize,
+    Clone,
+    PartialEq
+)]
 pub enum TestEvent {
     Created(Created),
     Tested(Tested),
     SomethingElse(SomethingElse),
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
+#[derive(
+    Debug,
+    Serialize,
+    Deserialize,
+    Clone,
+    PartialEq
+)]
 pub struct Created {
     pub id: String,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
+#[derive(
+    Debug,
+    Serialize,
+    Deserialize,
+    Clone,
+    PartialEq
+)]
 pub struct Tested {
     pub test_name: String,
 }
 
-#[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
+#[derive(
+    Debug,
+    Serialize,
+    Deserialize,
+    Clone,
+    PartialEq
+)]
 pub struct SomethingElse {
     pub description: String,
 }
 
-impl DomainEvent for TestEvent {
-    fn event_type(&self) -> String {
-        match self {
-            TestEvent::Created(_) => "Created".to_string(),
-            TestEvent::Tested(_) => "Tested".to_string(),
-            TestEvent::SomethingElse(_) => "SomethingElse".to_string(),
-        }
-    }
-
-    fn event_version(&self) -> String {
-        "1.0".to_string()
-    }
-}
+impl DomainEvent for TestEvent {}
 
 pub enum TestCommand {
     CreateTest(CreateTest),
@@ -164,13 +174,19 @@ struct TestView {
 }
 
 impl TestView {
-    fn new(events: Arc<RwLock<Vec<EventEnvelope<TestAggregate>>>>) -> Self {
+    fn new(
+        events: Arc<RwLock<Vec<EventEnvelope<TestAggregate>>>>
+    ) -> Self {
         TestView { events }
     }
 }
-#[async_trait]
-impl Query<TestAggregate> for TestView {
-    async fn dispatch(&self, _aggregate_id: &str, events: &[EventEnvelope<TestAggregate>]) {
+
+impl QueryProcessor<TestAggregate> for TestView {
+    fn dispatch(
+        &mut self,
+        _aggregate_id: &str,
+        events: &[EventEnvelope<TestAggregate>],
+    ) {
         for event in events {
             let mut event_list = self.events.write().unwrap();
             event_list.push(event.clone());
@@ -180,6 +196,11 @@ impl Query<TestAggregate> for TestView {
 
 pub type TestEventEnvelope = EventEnvelope<TestAggregate>;
 
+assert_impl_all!(aggregate; TestAggregate,Aggregate);
+// assert_impl_all!(event; TestEvent);
+
+assert_impl_all!(MemoryStore; MemoryStore::<TestAggregate>, EventStore::<TestAggregate,MemoryStoreAggregateContext<TestAggregate>>);
+
 fn metadata() -> HashMap<String, String> {
     let now = "2021-03-18T12:32:45.930Z".to_string();
     let mut metadata = HashMap::new();
@@ -187,13 +208,13 @@ fn metadata() -> HashMap<String, String> {
     metadata
 }
 
-#[tokio::test]
-async fn test_mem_store() {
-    let event_store = MemStore::<TestAggregate>::default();
+#[test]
+fn test_mem_store() {
+    let mut event_store = MemoryStore::<TestAggregate>::default();
     let id = "test_id_A";
-    let initial_events = event_store.load_events(&id).await.unwrap();
+    let initial_events = event_store.load(&id);
     assert_eq!(0, initial_events.len());
-    let agg_context = event_store.load_aggregate(&id).await.unwrap();
+    let agg_context = event_store.load_aggregate(&id);
 
     event_store
         .commit(
@@ -203,11 +224,10 @@ async fn test_mem_store() {
             agg_context,
             metadata(),
         )
-        .await
         .unwrap();
-    let stored_events = event_store.load_events(&id).await.unwrap();
+    let stored_events = event_store.load(&id);
     assert_eq!(1, stored_events.len());
-    let agg_context = event_store.load_aggregate(&id).await.unwrap();
+    let agg_context = event_store.load_aggregate(&id);
 
     event_store
         .commit(
@@ -219,20 +239,20 @@ async fn test_mem_store() {
                     test_name: "test B".to_string(),
                 }),
                 TestEvent::SomethingElse(SomethingElse {
-                    description: "something else happening here".to_string(),
+                    description: "something else happening here"
+                        .to_string(),
                 }),
             ],
             agg_context,
             metadata(),
         )
-        .await
         .unwrap();
-    let stored_envelopes = event_store.load_events(&id).await.unwrap();
+    let stored_envelopes = event_store.load(&id);
 
     let mut agg = TestAggregate::default();
     for stored_envelope in stored_envelopes {
         let event = stored_envelope.payload;
-        agg.apply(event);
+        agg.apply(&event);
     }
     println!("{:#?}", agg);
 }
@@ -242,8 +262,9 @@ type ThisTestFramework = TestFramework<TestAggregate>;
 #[test]
 fn test_framework_test() {
     let test_name = "test A";
+    let test_framework = ThisTestFramework::default();
 
-    ThisTestFramework::with(TestService)
+    test_framework
         .given(vec![TestEvent::Created(Created {
             id: "test_id_A".to_string(),
         })])
@@ -254,22 +275,23 @@ fn test_framework_test() {
             test_name: test_name.to_string(),
         })]);
 
-    ThisTestFramework::with(TestService)
+    test_framework
         .given(vec![TestEvent::Tested(Tested {
             test_name: test_name.to_string(),
         })])
         .when(TestCommand::ConfirmTest(ConfirmTest {
             test_name: test_name.to_string(),
         }))
-        .then_expect_error_message("test already performed")
+        .then_expect_error("test already performed")
 }
 
 #[test]
 #[should_panic]
 fn test_framework_failure_test() {
     let test_name = "test A";
+    let test_framework = ThisTestFramework::default();
 
-    ThisTestFramework::with(TestService)
+    test_framework
         .given(vec![TestEvent::Tested(Tested {
             test_name: test_name.to_string(),
         })])
@@ -285,26 +307,28 @@ fn test_framework_failure_test() {
 #[should_panic]
 fn test_framework_failure_test_b() {
     let test_name = "test A";
+    let test_framework = ThisTestFramework::default();
 
-    ThisTestFramework::with(TestService)
+    test_framework
         .given(vec![TestEvent::Created(Created {
             id: "test_id_A".to_string(),
         })])
         .when(TestCommand::ConfirmTest(ConfirmTest {
             test_name: test_name.to_string(),
         }))
-        .then_expect_error_message("some error message")
+        .then_expect_error("some error message")
 }
 
-#[tokio::test]
-async fn framework_test() {
-    let event_store = MemStore::default();
+#[test]
+fn framework_test() {
+    let event_store = MemoryStore::default();
     let stored_events = event_store.get_events();
 
     let delivered_events = Default::default();
     let view = TestView::new(Arc::clone(&delivered_events));
 
-    let cqrs = CqrsFramework::new(event_store, vec![Box::new(view)], TestService);
+    let mut cqrs =
+        CqrsFramework::new(event_store, vec![Box::new(view)]);
     let uuid = uuid::Uuid::new_v4().to_string();
     let id = uuid.clone();
     let metadata = metadata();
@@ -315,11 +339,13 @@ async fn framework_test() {
         }),
         metadata,
     )
-    .await
     .unwrap_or_default();
 
     assert_eq!(1, stored_events.read().unwrap().len());
-    assert_eq!(1, delivered_events.read().unwrap().len());
+    assert_eq!(
+        1,
+        delivered_events.read().unwrap().len()
+    );
 
     let test = "TEST_A";
     let id = uuid.clone();
@@ -329,10 +355,12 @@ async fn framework_test() {
             test_name: test.to_string(),
         }),
     )
-    .await
     .unwrap_or_default();
 
-    assert_eq!(2, delivered_events.read().unwrap().len());
+    assert_eq!(
+        2,
+        delivered_events.read().unwrap().len()
+    );
     let stored_event_count = stored_events
         .read()
         .unwrap()
@@ -349,16 +377,16 @@ async fn framework_test() {
                 test_name: test.to_string(),
             }),
         )
-        .await
         .unwrap_err();
-    match err {
-        AggregateError::UserError(payload) => {
-            assert_eq!("test already performed", payload.0.as_str())
-        }
-        _ => panic!("not the expected error"),
-    };
+    assert_eq!(
+        AggregateError::new("test already performed"),
+        err
+    );
 
-    assert_eq!(2, delivered_events.read().unwrap().len());
+    assert_eq!(
+        2,
+        delivered_events.read().unwrap().len()
+    );
     let stored_event_count = stored_events
         .read()
         .unwrap()
